#include "kappwidgetmanager.h"

#include <kappwidgetcommon.h>

#include <QDBusPendingReply>
#include <QDebug>
namespace AppWidget
{
KAppWidgetManager::KAppWidgetManager(QObject *parent)
    :QDBusInterface(KAppWidgetCommon::getManagerDbusObject(),
                    KAppWidgetCommon::getManagerDbusPath(),
                    KAppWidgetCommon::getManagerDbusInterface(),QDBusConnection::sessionBus(),parent)
{
    //! \todo init
}
QStringList KAppWidgetManager::getAppWidgetList()
{
    qDebug() << "KAppWidgetManager::getAppWidgetList";
    QStringList appWidgetList;
    QDBusMessage response = call("getAppWidgetList");
    if (response.type() == QDBusMessage::ErrorMessage ) {
        qCritical() <<  "ERROR" << response.errorMessage();
        return QStringList();
    }
    if (response.type() == QDBusMessage::ReplyMessage) {
       appWidgetList = response.arguments().takeFirst().toStringList();
    }
    return appWidgetList;
}
QString KAppWidgetManager::getAppWidgetUiFile(const QString &appwidgetname)
{
    qDebug() << "KAppWidgetManager::getAppWidgetUiFile";
    if(appwidgetname.isEmpty()) {
        qWarning() << "parameter is empty";
        return QString();
    }
    QString uiFile;
    QDBusMessage response = call("getAppWidgetUiFile", appwidgetname);
    if (response.type() == QDBusMessage::ErrorMessage ) {
        qCritical() <<  "ERROR" << response.errorMessage();
        return QString();
    }
    if (response.type() == QDBusMessage::ReplyMessage) {
       uiFile = response.arguments().takeFirst().toString();
    }
    return uiFile;
}
QString KAppWidgetManager::getAppWidgetConfig(const QString &appwidgetname,
                                                        const AppWidgetConfig& config)
{
    qDebug() << "KAppWidgetManager::getAppWidgetConfig";
    if(appwidgetname.isEmpty()) {
        qWarning() << "parameter is empty";
        return QString();
    }
    QString uiFile;
    QDBusMessage response = call("getAppWidgetConfig", appwidgetname, enumToSTring(config));
    if (response.type() == QDBusMessage::ErrorMessage ) {
        qCritical() <<  "ERROR" << response.errorMessage();
        return QString();
    }
    if (response.type() == QDBusMessage::ReplyMessage) {
       uiFile = response.arguments().takeFirst().toString();
    }
    return uiFile;
}
QMap<QString,QVariant> KAppWidgetManager::getAppWidgetConfigs(const QString &appwidgetname)
{
    qDebug() << "KAppWidgetManager::getAppWidgetConfigs";
    if(appwidgetname.isEmpty()) {
        qWarning() << "parameter is empty";
        return QMap<QString,QVariant>();
    }
    QMap<QString,QVariant> confs;
    QDBusPendingReply<QMap<QString,QVariant>> reply = call("getAppWidgetConfigs", appwidgetname);
    confs =reply.value();
    return confs;
}
int KAppWidgetManager::registerAppWidget(const QString &appwidgetname,
                                         const QString &username)
{
    qDebug() << "KAppWidgetManager::registerAppWidget";
    if(appwidgetname.isEmpty() || username.isEmpty()) {
        qWarning() << "parameter is empty";
        return -1;
    }
    int id = -1;
    QDBusMessage response = call("registerAppWidget", appwidgetname, username);
    if (response.type() == QDBusMessage::ErrorMessage ) {
        qCritical() <<  "ERROR" << response.errorMessage();
        return -1;
    }
    if (response.type() == QDBusMessage::ReplyMessage) {
       id = response.arguments().takeFirst().toInt();
    }
    return id;
}
bool KAppWidgetManager::unregisterAppWidget(const int &appwidgetid)
{
    qDebug() << "KAppWidgetManager::unregisterAppWidget";
    if(appwidgetid < 0) {
        qWarning() << "parameter is error";
        return false;
    }
    bool unregister = false;
    QDBusMessage response = call("unregisterAppWidget", appwidgetid);
    if (response.type() == QDBusMessage::ErrorMessage ) {
        qCritical() << __FUNCTION__ << __LINE__ << response.errorMessage();
        return false;
    }
    if (response.type() == QDBusMessage::ReplyMessage) {
        unregister = response.arguments().takeFirst().toBool();
    }
    return unregister;
}
QMap<QString, QVariant> KAppWidgetManager::getAppWidgetData(const int &appwidgetid)
{
    qDebug() << "KAppWidgetManager::getAppWidgetData";
    if(appwidgetid < 0) {
        qWarning() << __FUNCTION__ << __LINE__ << "get invalid id :" << appwidgetid;
        return QMap<QString, QVariant>();
    }
    QDBusPendingReply<QMap<QString, QVariant>> reply = asyncCall("getAppWidgetData", appwidgetid);
    reply.waitForFinished();
    if (reply.isError()) {
        qDebug() << __FUNCTION__ << __LINE__ << reply.error().message();
        return QMap<QString, QVariant>();
    }
    QMap<QString, QVariant> map = reply.value();
    return map;
}
void KAppWidgetManager::updateAppWidget(const QString &providername,
                                        const QMap<QString, QVariant> &data)
{
    qDebug() << "KAppWidgetManager::updateAppWidget" << providername << data;
    if(providername.isEmpty()) {
        qWarning() << "parameter is empty";
        return ;
    }
    asyncCall("updateAppWidget", providername, data);
}
QString KAppWidgetManager::enumToSTring(const AppWidgetConfig& config)
{
    QString conf;
    switch (config) {
    case AppWidgetConfig::NAME:
        conf = "name";
        break;
    case AppWidgetConfig::PROVIDER_NAME:
        conf = "providerName";
        break;
    case AppWidgetConfig::PREVIEW_PATH:
        conf = "previewPath";
        break;
    case AppWidgetConfig::ZOOM:
        conf = "zoom";
        break;
    case AppWidgetConfig::RIGHT_BUTTON:
        conf = "rightButton";
        break;
    case AppWidgetConfig::REPEAT_PLACEMENT:
        conf = "repeatPlacement";
        break;
    case AppWidgetConfig::MIN_WIDTH:
        conf = "minWidget";
        break;
    case AppWidgetConfig::MIN_HEIGHT:
        conf = "minHeight";
        break;
    case AppWidgetConfig::TARGET_CELL_WIDTH:
        conf = "targetCellWidth";
        break;
    case AppWidgetConfig::TARGET_CELL_HEIGHT:
        conf = "targetCellHeight";
        break;
    case AppWidgetConfig::UPDATE_PERIODMILLIS:
        conf = "updatePeriodMillis";
        break;
    case AppWidgetConfig::DESCRIBE:
        conf = "describe";
        break;
    default:
        break;
    }
    return conf;
}
}// namespace AppWidget
