#include "kappwidgetconnect.h"
#include <kappwidgetmanager.h>
#include <kappwidgetcommon.h>

#include <QDBusPendingReply>
#include <QDebug>

using namespace AppWidget;
KAppWidgetConnect::KAppWidgetConnect(QObject *parent) : QObject(parent)
{
}

KAppWidgetConnect::~KAppWidgetConnect()
{
    appwidgetinterface->unregisterAppWidget(m_id);
}

void KAppWidgetConnect::onClickedEvent(const QString &widgetname, const QVariant &value)
{
    qDebug() << __FILE__ << __FUNCTION__ <<  widgetname << " send clicked signal";
    QDBusVariant variant;
    variant.setVariant(value);
    Q_EMIT clickedSignal(widgetname, variant);
}

void KAppWidgetConnect::onDefineEvent(const QString &eventname, const QString &widgetname, const QVariant &value)
{
    qDebug() << __FILE__ << __FUNCTION__ << widgetname << "Send DefineEvent Signal" << value;
    QDBusVariant variant;
    variant.setVariant(value);
    Q_EMIT defineEventSignal(eventname, widgetname, variant);
}

void KAppWidgetConnect::registerDbus(const QString &appname, const QString &username)
{
    qDebug() << __FILE__ << __FUNCTION__ << "Register Appwidget Service: " << username << "," << appname;
    m_appname = appname;
    m_username = username;

    appwidgetinterface = new KAppWidgetManager(this);
    m_id = appwidgetinterface->registerAppWidget(m_appname, m_username);
    qDebug() << __FUNCTION__ << __LINE__ << "id: " << m_id;
    QString object = KAppWidgetCommon::getAppWidgetDbusObject(m_username, m_id);
    QString objectpath = KAppWidgetCommon::getAppWidgetDbusPath(m_username, m_id);
    QDBusConnection sessionBus = QDBusConnection::sessionBus();
    if (sessionBus.registerService(object)) {
        qDebug() << "Regist Dbus Service success";
        sessionBus.registerObject(objectpath, this, QDBusConnection :: ExportAllSlots
                                  | QDBusConnection :: ExportAllSignals
                                  | QDBusConnection::ExportAllInvokables);
    } else {
        qCritical() << "Register DBus Service Error:" << sessionBus.lastError().message();
    }
}
QMap<QString, QVariant> KAppWidgetConnect::getData()
{
    QMap<QString, QVariant> reply = appwidgetinterface->getAppWidgetData(m_id);
    qDebug() << __FILE__ << __FUNCTION__ << ": " << reply;
    return reply;
}
void KAppWidgetConnect::updateAppWidget()
{
    Q_EMIT updatesignal();
}
