#ifndef APPWIDGETMANAGER_H
#define APPWIDGETMANAGER_H

#include <QObject>
#include <QDBusVariant>

namespace AppWidget {
class AppWidgetProviderProxy;
class AppWidgetProxy;
class MappingManager;
class AppWidgetConfiguration;
class AppWidgetManager : public QObject
{
    Q_OBJECT

public:
    explicit AppWidgetManager(QObject *parent = nullptr);
    ~AppWidgetManager();

public:
    //定时更新
    //映射管理
    MappingManager * mappingmanager = nullptr;
    //小插件列表管理
    //小插件配置管理
    AppWidgetConfiguration * configmanager = nullptr;

public slots:
    QStringList getAppWidgetList();
    QString getAppWidgetUiFile(QString appwidgetname);
    QString getAppWidgetConfig(QString appwidgetname, QString config);
    QMap<QString,QVariant> getAppWidgetConfigs(QString appwidgetname);
    int registerAppWidget(QString appwidgetname, QString username);
    bool unregisterAppWidget(int appwidgetid);
    QMap<QString, QVariant> getAppWidgetData(int appwidgetid);
    void updateAppWidget(QString providername, QMap<QString, QVariant> data);

signals:
    void appWidgetFileAdded(QString appwidgetname);
    void appWidgetFileDeleted (QString appwidgetname);
    //void appWidgetDataChanged(int ids);

private:
    void init();
    void registerCommunicationService();//注册dbus服务
    bool registerScheduledUpdate(AppWidgetProviderProxy* providerproxy, int timingperiod);//注册定时更新
    bool unregisterScheduledUpdate(AppWidgetProviderProxy* providerproxy);//注销定时更新
    AppWidgetProviderProxy* createProviderProxy(QString providername);
    void deleteProviderProxy(AppWidgetProviderProxy* providerproxy);
    AppWidgetProxy* createAppWidgetProxy(AppWidgetProviderProxy* providerproxy, QString appwidgetname, QString username);
    bool deleteAppWidgetProxy(AppWidgetProviderProxy* providerproxy, int id);
};
} // namespace AppWidget
#endif // APPWIDGETMANAGER_H
